 /**
 * Class for drawing a background image with canvas
 *
 * This class handles the updating from the background 
 * image from a html element from an original source.
 * @constructor
 * @author Christian Krebs, Opera Software ASA
 */

HTMLElement.BGImage=function(ref, url)
{
  var refEle=ref;
  var img=new Image();
  var ready=false;
  var isnotdrawing=true;
  var queueBorders=null;
  var queueDraw=0;
  var top=0;
  var right=0;
  var bottom=0;
  var left=0;
  var width=0;
  var height=0;
  var sourceWidth=0
  var sourceHeight=0;
  var sourceCenterWidth=0;
  var sourceCenterHeight=0;
  var canvas=document.createElement('canvas');
  var ctx=canvas.getContext('2d');
  var canvas_t=document.createElement('canvas');
  var ctx_t=canvas_t.getContext('2d');
  var cx=0, cy=0, ch=top, scy=0;
  var self=this;
  var resetIsnotdrawing=function()
  {
    isnotdrawing=true;
    if(queueDraw)
    {
      _draw();
      queueDraw=0;
    }
  }
  var drawLine=function(cctx)
  {
    cctx.drawImage(img, cx, scy, left, ch, 0, cy, left, ch);
    cx=left;
    while(cx<width-right)
    {
      cctx.drawImage(img, left, scy, sourceCenterWidth, ch, cx, cy , sourceCenterWidth, ch);
      cx+=sourceCenterWidth;
    }
    cx=width-right;
    cctx.clearRect(cx, cy, right, ch);
    cctx.drawImage(img, sourceWidth-right, scy, right, ch, cx, cy , right, ch);
  }
  var imgLoadHandler=function()
  {
    sourceWidth=this.width;
    sourceHeight=this.height;
    ready=true;
    if(queueBorders)
    {
      _setBorders.apply(null, queueBorders);
      queueBorders=null;
    }
    if(queueDraw)
    {
      _draw();
      queueDraw=0;
    }
  }
  var _setBorders=function()
  {
    switch(arguments.length)
    {
      case 1:
        top=right=bottom=left=arguments[0];
        break;
      case 2:
        top=bottom=arguments[0];
        right=left=arguments[1];
        break;
      case 4:
        top=arguments[0];
        right=arguments[1];
        bottom=arguments[2];
        left=arguments[3];
        break;
    } 
    sourceCenterWidth=sourceWidth-right-left;
    sourceCenterHeight=sourceHeight-top-bottom;
  }
  var _draw=function()
  {
    isnotdrawing=false;
    width=parseInt(refEle.offsetWidth);
    height=parseInt(refEle.offsetHeight);
    canvas.width=width;
    canvas.height=height;
    // workaround for missing alpha channel in toDataURL()
    canvas.style.width=width+'px';
    canvas.style.height=height+'px';
    if(!canvas.style.position)
    {
      canvas.style.position='absolute';
      canvas.style.left=refEle.style.left;
      canvas.style.top=refEle.style.top;
      refEle.parentNode.appendChild(canvas);
      var zIndex=parseInt(refEle.currentStyle.zIndex);
      if(!zIndex)
      {
        refEle.style.zIndex=1;
        canvas.style.zIndex=0;
      }
      else
      {
        canvas.style.zIndex=zIndex-1;
      }
    }
    //
    cx=0;
    cy=0;
    ch=top;
    scy=0;
    ctx.clearRect(cx,cy,width,height);
    drawLine(ctx);
    canvas_t.width=width;
    canvas_t.height=sourceCenterHeight;
    ctx_t.clearRect(0,0,width,sourceCenterHeight); 
    cx=0;
    scy=top;
    ch=sourceCenterHeight;
    drawLine(ctx_t);
    cy=top;
    while(cy<height-bottom)
    {
      ctx.drawImage(canvas_t, 0, 0, width, ch, 0,cy,width,ch);
      cy+=sourceCenterHeight;
    }
    ctx.clearRect(0,height-bottom,width, bottom);
    cx=0;
    cy=height-bottom;
    scy=sourceHeight-bottom;
    ch=bottom;
    drawLine(ctx);
    // workaround for missing alpha channel in toDataURL()
    //refEle.style.background='transparent url('+canvas.toDataURL()+') scroll no-repeat 0 0';
    setTimeout(resetIsnotdrawing, 15);
  }
  /**
   * To set the borders on the source image to define 
   * the different tiles (topLeft, top, topRight, left, center, right, bottomLeft, bottom bottomRight)
   * @arg borders in css logic
   */
  this.setBorders=function()
  {
    if(ready)
    {
      _setBorders.apply(null, arguments);
    }
    else
    {
      queueBorders=arguments;
    }
  }
  /**
   * To update the background image.
   */
  this.draw=function()
  {
    if(ready && isnotdrawing) _draw();
    else queueDraw=1;
  }
  /**
   * @private
   */
  this.cleanup=function(event)
  {
    if(event.target==refEle)
    {
      this.removeEventListener('DOMNodeRemoved', this.bgImage.cleanup, false);
      canvas.parentNode.removeChild(canvas);
      delete this.bgImage;
      refEle=null;
      queueBorders=null;
      img=null;
      canvas=null;
      ctx=null;
      canvas_t=null;
      ctx_t=null;
    }
  }
  img.onload=imgLoadHandler;
  img.src=url;
}
/**
 * To create a background image updater.
 * @arg url for the original image.
 * @returns a background image updater as property 'bgImage' on the HTMLElement.
 */
HTMLElement.prototype.createBGImage=function(url)
{
  this.bgImage=new HTMLElement.BGImage(this, url);
  this.addEventListener('DOMNodeRemoved', this.bgImage.cleanup, false);
}