/* Florence - Feed Library by Nicolas Mendoza */
/* TODO: rigid rss/atom support, store, subscription, formatting */

var ASC = 0; // Ascending order of feeds
var DESC = 1; // Descending order of feeds
var FEED_ORDER = ASC;


var XHR = 1;
var XHR_SOP = 2;
var JS = 3;

var FETCH_METHOD = XHR; // For some reason the XML/JS check further down (navigator.platform) didn't work on N70, we force it for all devices.

var JS_PROXY = "";
var XHR_SO_PROXY = "";

/* Feed class */

var feedStore = new FeedStore();

function Feed() {
  this.items = new Array();
  this.req = null;
  this.state = 1;
  this.feedReadyHandler = null;
  this.feedMessageHandler = alert;
  this.feedErrorHandler = alert;
  this.feedTimeOutHandler = null;
  this.processItemCallBack = null;
  this.error = null;
  this.itemize = null;
  this.timeOutID = null;

  var self = this;
  // url to fetch feed from
  this.url = null;
  // seconds between updates, defaults to an hour
  this.interval = 60;
  this.timeout = 10;

  feedStore.feeds.push(this);

  this.setFeedReadyHandler = function(func) {
    this.feedReadyHandler = func;
  }
  
  this.setFeedErrorHandler = function(func) {
    this.feedErrorHandler = func;
  }
  
  this.setFeedMessageHandler = function(func) {
    this.feedMessageHandler = func;
  }
  
  this.setProcessItemCallBack = function(func) {
    this.processItemCallBack = func;
  }
  
  this.refresh = function() {
    self.sendRequest();
    setTimeout(self.refresh,self.interval*1000*60);
  }

  this.sendRequest = function(){
    this.timeoutID = window.setTimeout(function() { self.stopRequest(); if (self.feedTimeOutHandler) { self.feedTimeOutHandler(self); } } ,self.timeout*1000);
    this.feedMessageHandler("sending request");
    switch(FETCH_METHOD) {
      case XHR:
        this.feedMessageHandler("Trying XHR");
        if (this.sendXMLRequest(true)) {
          break;
        } else {
          FETCH_METHOD = XHR_SOP;
        }
      case XHR_SOP:
        this.feedMessageHandler("Trying XHR same origin proxy");    
        if (this.sendXMLRequest(true)) {
          break;
        } else {
          FETCH_METHOD = JS;
        }        
      case JS:
        this.feedMessageHandler("Trying JS proxy");        
        this.sendJSRequest();
      default:
        break;
      }
  }

  this.stopRequest = function() {
    window.clearTimeout(this.timeoutID);
    this.feedMessageHandler("stopping request");
    switch(FETCH_METHOD) {
      case XHR:
      case XHR_SOP:
        this.feedMessageHandler("Stopping XHR");
        this.stopXMLRequest();
      case JS:
        this.feedMessageHandler("Stopping JS");        
        this.stopJSRequest();
      default:
        break;
      }       
  }

  this.sendXMLRequest = function(silent) {
    this.stopXMLRequest();
    var url;
    if (!window.XMLHttpRequest) { return 0; }
    if (FETCH_METHOD == XHR_SOP) { url = XHR_SO_PROXY + escape(this.url); } else { url = this.url; }
    this.req = new XMLHttpRequest();
    this.req.onreadystatechange = function() { self.processRequest(self); }
    this.req.open("GET", url, true);
    try {
      this.req.send(null);
    } catch (e) {
      if (!silent) {
        this.feedErrorHandler("Unable to fetch news: e.message");
      }
      return 0;
    }
    return 1;
  }
  
  this.stopXMLRequest = function() {
    if (this.req && this.req.abort) {
      this.req.abort();
    }
  }
    
  this.sendJSRequest = function() {
    this.stopJSRequest();
    this.feedMessageHandler("Creating JS element")
    var jsElement = document.createElement("script");
    jsElement.type = "text/javascript";
    this.feedMessageHandler("Building URL");
    jsElement.src = JS_PROXY+escape(this.url);
    // TODO: Set proper id.
    jsElement.id = "foo";
    this.id = jsElement.id;
    var tmpURL = this.url;
    this.feedMessageHandler("Setting up callback")
    window.setContent = function() {
      for(var i=0; i<feedStore.feeds.length; i++) {
        if (feedStore.feeds[i].url == tmpURL) {
       	 feedStore.feeds[i].responseText = window.content;
       	 if (!feedStore.feeds[i].responseText) { feedStore.feeds[i].feedMessageHandler("No content!");}
          if (feedStore.feeds[i].updateItems()) {
            //alert(self.responseText);
            feedStore.feeds[i].feedMessageHandler("clearing timeout");
            window.clearTimeout(feedStore.feeds[i].timeoutID);
            feedStore.feeds[i].feedReadyHandler(feedStore.feeds[i]);
          } else {
            feedStore.feeds[i].feedErrorHandler("Not able to update items");
          }
          //FIXME: we shouldn't have several identical feeds in feedStore.
          break;   
        } 
     } 
    };    
    
    this.feedMessageHandler("Getting head")
    var head = document.getElementsByTagName("head")[0];
    if (window.content) {
      this.feedMessageHandler("Removing previous items");
      head.removeChild(head.childNodes[head.childNodes.length-1]);
    }
    this.feedMessageHandler(jsElement.src);
    this.feedMessageHandler("Adding JS ele to head")
    head.appendChild(jsElement);
  }  
  
  this.stopJSRequest = function() {
    var jsElement = document.getElementById(this.id);
    if (jsElement) {
      jsElement.parentNode.removeChild(jsElement);
    }
  }  
  
  this.processRequest = function(feed) {
    // only if req shows "loaded"
    //opera.postError(feed.req.readyState);
    if (feed.req.readyState == 4) {
      // only if "OK"
      if (feed.req.status == 200 || feed.req.status == 0) {
  		  if (feed.req.responseXML || feed.req.responseText) {
	  	    //alert("foo");
	  	      // avoid probable bug with empty response if request is cached.
            feed.responseText = feed.req.responseText;
            this.feedMessageHandler("clearing timeout");
            window.clearTimeout(this.timeoutID);
		      if (feed.updateItems()) {
  		        if (feed.feedReadyHandler) {
                feed.feedReadyHandler(feed);
	    		  }
	    		} else {
             this.feedErrorHandler("Not able to update items");
          }
	  	  }
      } else {
        feed.feedErrorHandler("There was a problem retrieving the XML data:\n" +
                feed.req.statusText+"\n"+feed.url+"\n"+feed.req.status+" \n"+feed.req.responseText+"\n"+feed.req.getAllResponseHeaders());
      }
    }
  }
  
  this.updateItems = function() {
    //alert("bar: "+this.responseText);
    if (FETCH_METHOD != JS) {
      //alert("foo");
      this.responseText = this.req.responseText;
    }

      // TODO: Add items, instead of replace.
      // Not valid XML ;(
      this.feedMessageHandler("Updating items");
      
      if (this.itemize()) {
        return true;
      } else {
        this.feedErrorHandler("Not able to fetch URL: "+this.url+"\n\n"+this.responseText);
        return false;
      }
/*      var tmpItems = this.responseText.match(/<item>[\s\S]*?<\/item>/mgi);

      if (tmpItems) {
        this.feedMessageHandler("Got "+tmpItems.length+" items");s
        this.items = [];
        for(var i=0;i<tmpItems.length;i++) {
          this.items[i] = {};
          this.items[i].outerHTML = tmpItems[i];
          this.items[i].title = tmpItems[i].match(/<title>([\s\S]*?)<\/title>/mi)[1];
          this.items[i].description = 
tmpItems[i].match(/<description>[\s\S]*?([\s\S]*?)<\/description>/mi)[1].replace(/^<!\[CDATA\[([\s\S]*?)\]\]>$/,"$1");
          this.items[i].pubDate = tmpItems[i].match(/<pubDate[\s\S]*?>([\s\S]*?)<\/pubDate>/mi)[1];
          this.items[i].link = tmpItems[i].match(/<link[\s\S]*?>([\s\S]*?)<\/link>/mi)[1];
          this.items[i].guid = tmpItems[i].match(/<guid[\s\S]*?>([\s\S]*?)<\/guid>/mi)[1];
          if (this.processItemCallBack) {
            this.items[i] = this.processItemCallBack(this.items[i]);
          }      
        }
        return true;
      } 

        this.feedErrorHandler("Not able to fetch URL: "+this.url+"\n\n"+this.responseText);
        return false;  
        */
   }
}

/* FeedStore class */

function FeedStore() {
  this.feeds = [];

  this.refresh = function() {
	 for(var i=0;i<feeds.length;i++) {
	 	feeds[i].fetchContent;
	 }
  }

  this.addFeed = function(feed) {
    feeds[feeds.size] = feed;
  }

  this.removeFeed = function(feed) {
    for(var i=0;i<feeds.length;i++) {
    	if(feeds[i] === feed) {
    	   feeds.splice(i,1);
    	}
    }
  }  
}

/* Helper functions */

function unescapeHTML(string) {
  var div = document.createElement('div');
  div.innerHTML = string.replace(/<\/?[^>]+>/gi, '');
  return div.childNodes[0] ? div.childNodes[0].nodeValue : '';
}

function dumpObject(obj) {
  var b;
  for (a in obj) {
    b+= a+"\n";
  }
  opera.postError(b);
}

function echo(string) {
	    document.getElementsByTagName("body")[0].innerHTML += string;  
}  

/* End */
